@@ -31,7 +31,7 @@ from gettext import gettext as _
 from gettext import ngettext
 
 import pitivi.ui.dnd as dnd
-from pitivi.ui.pathwalker import PathWalker
+from pitivi.ui.pathwalker import PathWalker, quote_uri
 from pitivi.ui.filelisterrordialog import FileListErrorDialog
 from pitivi.configure import get_pixmap_dir
 from pitivi.signalgroup import SignalGroup
@@ -44,6 +44,9 @@ from pitivi.ui.common import beautify_factory, factory_name, \
 from pitivi.log.loggable import Loggable
 from pitivi.sourcelist import SourceListError
 
+SHOW_TREEVIEW = 1
+SHOW_ICONVIEW = 2
+
 GlobalSettings.addConfigSection('clip-library')
 GlobalSettings.addConfigOption('lastImportFolder',
     section='clip-library',
@@ -54,13 +57,20 @@ GlobalSettings.addConfigOption('closeImportDialog',
     section='clip-library',
     key='close-import-dialog-after-import',
     default=True)
+GlobalSettings.addConfigOption('lastClipView',
+    section='clip-library',
+    key='last-clip-view',
+    type_=int,
+    default=SHOW_ICONVIEW)
 
 (COL_ICON,
+ COL_ICON_LARGE,
  COL_INFOTEXT,
  COL_FACTORY,
  COL_URI,
  COL_LENGTH,
- COL_SEARCH_TEXT) = range(6)
+ COL_SEARCH_TEXT,
+ COL_SHORT_TEXT) = range(8)
 
 ui = '''
 <ui>
@@ -99,38 +109,43 @@ class SourceList(gtk.VBox, Loggable):
         Loggable.__init__(self)
 
         self.app = instance
+        self.settings = instance.settings
 
         # Store
         # icon, infotext, objectfactory, uri, length
-        self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, object, str, str,
-            str)
+        self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, gtk.gdk.Pixbuf, str, object, str, str,
+            str, str)
 
-        # Scrolled Window
-        self.scrollwin = gtk.ScrolledWindow()
-        self.scrollwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-        self.scrollwin.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+        # Scrolled Windows
+        self.treeview_scrollwin = gtk.ScrolledWindow()
+        self.treeview_scrollwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        self.treeview_scrollwin.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+
+        self.iconview_scrollwin = gtk.ScrolledWindow()
+        self.iconview_scrollwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.iconview_scrollwin.set_shadow_type(gtk.SHADOW_ETCHED_IN)
 
         # Popup Menu
         self.popup = gtk.Menu()
-        additem = gtk.ImageMenuItem(_("Add Clips..."))
+        self.popup_importitem = gtk.ImageMenuItem(_("Import clips..."))
         image = gtk.Image()
         image.set_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
-        additem.set_image(image)
+        self.popup_importitem.set_image(image)
 
-        remitem = gtk.ImageMenuItem(_("Remove Clip"))
+        self.popup_remitem = gtk.ImageMenuItem(_("Remove Clip"))
         image = gtk.Image()
         image.set_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
-        remitem.set_image(image)
-        playmenuitem = gtk.MenuItem(_("Play Clip"))
-        playmenuitem.connect("activate", self._playButtonClickedCb)
-        additem.connect("activate", self._addButtonClickedCb)
-        remitem.connect("activate", self._removeButtonClickedCb)
-        additem.show()
-        remitem.show()
-        playmenuitem.show()
-        self.popup.append(additem)
-        self.popup.append(remitem)
-        self.popup.append(playmenuitem)
+        self.popup_remitem.set_image(image)
+        self.popup_playmenuitem = gtk.MenuItem(_("Play Clip"))
+        self.popup_importitem.connect("activate", self._importButtonClickedCb)
+        self.popup_remitem.connect("activate", self._removeButtonClickedCb)
+        self.popup_playmenuitem.connect("activate", self._playButtonClickedCb)
+        self.popup_importitem.show()
+        self.popup_remitem.show()
+        self.popup_playmenuitem.show()
+        self.popup.append(self.popup_importitem)
+        self.popup.append(self.popup_remitem)
+        self.popup.append(self.popup_playmenuitem)
 
         # import sources dialogbox
         self._importDialog = None
@@ -138,6 +153,7 @@ class SourceList(gtk.VBox, Loggable):
         # TreeView
         # Displays icon, name, type, length
         self.treeview = gtk.TreeView(self.storemodel)
+        self.treeview_scrollwin.add(self.treeview)
         self.treeview.connect("button-press-event", self._treeViewButtonPressEventCb)
         self.treeview.connect("row-activated", self._rowActivatedCb)
         self.treeview.set_property("rules_hint", True)
@@ -145,7 +161,7 @@ class SourceList(gtk.VBox, Loggable):
         self.treeview.set_property("search_column", COL_SEARCH_TEXT)
         tsel = self.treeview.get_selection()
         tsel.set_mode(gtk.SELECTION_MULTIPLE)
-        tsel.connect("changed", self._treeSelectionChanged)
+        tsel.connect("changed", self._viewSelectionChangedCb)
 
         pixbufcol = gtk.TreeViewColumn(_("Icon"))
         pixbufcol.set_expand(False)
@@ -175,11 +191,20 @@ class SourceList(gtk.VBox, Loggable):
         namecol.pack_start(txtcell)
         namecol.add_attribute(txtcell, "markup", COL_LENGTH)
 
-        # Start up with tree view
-        self.scrollwin.add(self.treeview)
+        # IconView
+        self.iconview = gtk.IconView(self.storemodel)
+        self.iconview_scrollwin.add(self.iconview)
+        self.iconview.connect("button-press-event", self._iconViewButtonPressEventCb)
+        self.iconview.connect("selection-changed", self._viewSelectionChangedCb)
+        self.iconview.set_orientation(gtk.ORIENTATION_VERTICAL)
+        self.iconview.set_text_column(COL_SHORT_TEXT)
+        self.iconview.set_pixbuf_column(COL_ICON_LARGE)
+        self.iconview.set_selection_mode(gtk.SELECTION_MULTIPLE)
+        self.iconview.set_item_width(106)
 
         # Explanatory message label
         textbox = gtk.EventBox()
+        textbox.connect("button-press-event", self._textBoxButtonPressEventCb)
         textbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('white'))
         textbox.show()
 
@@ -192,17 +217,11 @@ class SourceList(gtk.VBox, Loggable):
             _("<span size='x-large'>Import your clips by dragging them here or "
               "by using the buttons above.</span>"))
         textbox.add(txtlabel)
+        txtlabel.show()
         self.txtlabel = txtlabel
 
         self.textbox = textbox
 
-        self.pack_start(self.textbox, expand=True, fill=True)
-        self.reorder_child(self.textbox, 0)
-        self.showingTreeView = False
-
-        self.dragMotionSigId = self.txtlabel.connect("drag-motion",
-                                                     self._dragMotionCb)
-
         self.infostub = InfoStub()
         self.infostub.connect("remove-me", self._removeInfoStub)
 
@@ -231,9 +250,17 @@ class SourceList(gtk.VBox, Loggable):
             self._treeViewMotionNotifyEventCb)
         self.treeview.connect("button-release-event",
             self._treeViewButtonReleaseCb)
-        self.treeview.connect("drag_begin", self._dndTreeBeginCb)
+        self.treeview.connect("drag_begin", self._dndDragBeginCb)
         self.treeview.connect("drag_data_get", self._dndDataGetCb)
 
+        self.iconview.drag_source_set(0,[], gtk.gdk.ACTION_COPY)
+        self.iconview.connect("motion-notify-event",
+            self._iconViewMotionNotifyEventCb)
+        self.iconview.connect("button-release-event",
+            self._iconViewButtonReleaseCb)
+        self.iconview.connect("drag_begin", self._dndDragBeginCb)
+        self.iconview.connect("drag_data_get", self._dndDataGetCb)
+
         # Hack so that the views have the same method as self
         self.treeview.getSelectedItems = self.getSelectedItems
 
@@ -252,7 +279,7 @@ class SourceList(gtk.VBox, Loggable):
         # only available when selection is non-empty 
         selection_actions = (
             ("RemoveSources", gtk.STOCK_DELETE,
-                _("_Remove from project"), None, None,
+                _("_Remove from project"), "<Control>Delete", None,
                 self._removeSourcesCb),
             ("InsertEnd", gtk.STOCK_COPY,
                 _("Insert at _end of timeline"), "Insert", None,
@@ -270,6 +297,45 @@ class SourceList(gtk.VBox, Loggable):
         uiman.insert_action_group(self.selection_actions, 0)
         uiman.add_ui_from_string(ui)
 
+        # clip view menu items
+        view_menu_item = uiman.get_widget('/MainMenuBar/View')
+        view_menu = view_menu_item.get_submenu()
+        seperator = gtk.SeparatorMenuItem()
+        self.treeview_menuitem = gtk.RadioMenuItem(None,
+                "Show Clips as a List")
+        self.iconview_menuitem = gtk.RadioMenuItem(self.treeview_menuitem,
+                "Show Clips as Icons")
+
+        # update menu items with current clip view before we connect to item
+        # signals
+        if self.settings.lastClipView == SHOW_TREEVIEW:
+            self.treeview_menuitem.set_active(True)
+            self.iconview_menuitem.set_active(False)
+        else:
+            self.treeview_menuitem.set_active(False)
+            self.iconview_menuitem.set_active(True)
+
+        # we only need to connect to one menu item because we get a signal
+        # from each radio item in the group
+        self.treeview_menuitem.connect("toggled", self._treeViewMenuItemToggledCb)
+
+        view_menu.append(seperator)
+        view_menu.append(self.treeview_menuitem)
+        view_menu.append(self.iconview_menuitem)
+        self.treeview_menuitem.show()
+        self.iconview_menuitem.show()
+        seperator.show()
+
+        # add all child widgets
+        self.pack_start(self.textbox, expand=True, fill=True)
+        self.pack_start(self.iconview_scrollwin)
+        self.pack_start(self.treeview_scrollwin)
+
+        # display the help text
+        self.showing_helptext = None
+        self.clip_view = self.settings.lastClipView
+        self._displayHelpText()
+
     def _importSourcesCb(self, unused_action):
         self.showImportSourcesDialog()
 
@@ -284,6 +350,7 @@ class SourceList(gtk.VBox, Loggable):
         timeline = self.app.current.timeline
         sources = self.app.current.sources
         start = timeline.duration
+        self.app.current.seeker.seek(start)
         for uri in self.getSelectedItems():
             factory = sources.getUri(uri)
             source = timeline.addSourceFactory(factory)
@@ -326,43 +393,53 @@ class SourceList(gtk.VBox, Loggable):
 
 
     ## Explanatory message methods
+    
+    def _setClipView(self, show):
+        """ Set which clip view to use when sourcelist is showing clips. If
+        none is given, the current one is used. Show: one of SHOW_TREEVIEW or
+        SHOW_ICONVIEW """
 
-    def _displayTreeView(self, displayed=True, usesignals=True):
-        """ Display the tree view in the scrolled window.
-        If displayed is False, then the default explanation message will be
-        shown.
-        If usesignals is True, then signals on the mesagewindow will be
-        (dis)connected
-        """
-        if displayed:
-            if self.showingTreeView:
-                return
+        # save current selection
+        paths = self.getSelectedPaths()
+
+        # update saved clip view
+        self.settings.lastClipView = show
+        self.clip_view = show
+
+        # transfer selection to next view
+        self._viewUnselectAll()
+        for path in paths:
+            self._viewSelectPath(path)
+
+        # if we're not in help text mode, switch to the new clipview
+        if not self.showing_helptext:
+            self._displayClipView()
+
+    def _displayClipView(self):
+
+        # first hide all the child widgets
+        self.showing_helptext = False
+        self.textbox.hide()
+        self.treeview_scrollwin.hide()
+        self.iconview_scrollwin.hide()
+
+        # pick the widget we're actually showing
+        if self.clip_view == SHOW_TREEVIEW:
             self.debug("displaying tree view")
-            self.remove(self.textbox)
-            self.txtlabel.hide()
-            if usesignals:
-                if self.dragMotionSigId:
-                    self.txtlabel.disconnect(self.dragMotionSigId)
-                    self.dragMotionSigId = 0
-            self.pack_start(self.scrollwin)
-            self.reorder_child(self.scrollwin, 0)
-            self.scrollwin.show_all()
-            self.showingTreeView = True
-        else:
-            if not self.showingTreeView:
-                return
-            self.debug("hiding tree view")
-            self.remove(self.scrollwin)
-            self.scrollwin.hide()
-            self.pack_start(self.textbox)
-            self.reorder_child(self.textbox, 0)
-            self.txtlabel.show()
-            self.showingTreeView = False
+            widget = self.treeview_scrollwin
+        elif self.clip_view == SHOW_ICONVIEW:
+            self.debug("displaying icon view")
+            widget = self.iconview_scrollwin
 
-    def _dragMotionCb(self, unused_layout, unused_context, unused_x, unused_y,
-                      unused_timestamp):
-        self.log("motion")
-        gobject.idle_add(self._displayTreeView, True, False)
+        # now un-hide the view
+        widget.show_all()
+
+    def _displayHelpText(self):
+        """Hides the current clip view and displays the help text"""
+        self.textbox.show()
+        self.iconview_scrollwin.hide()
+        self.treeview_scrollwin.hide()
+        self.showing_helptext = True
 
     def showImportSourcesDialog(self, select_folders=False):
         """Pop up the "Import Sources" dialog box"""
@@ -382,6 +459,7 @@ class SourceList(gtk.VBox, Loggable):
                                                    chooser_action,
                                                    (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE,
                                                     gtk.STOCK_ADD, gtk.RESPONSE_OK))
+        self._importDialog.set_icon_name("pitivi")
         self._importDialog.props.extra_widget = close_after
         self._importDialog.set_default_response(gtk.RESPONSE_OK)
         self._importDialog.set_select_multiple(True)
@@ -412,28 +490,46 @@ class SourceList(gtk.VBox, Loggable):
                 self.error("Failure to create thumbnail from file '%s'",
                         thumbnail_file)
                 thumbnail = self.videofilepixbuf
+                thumbnail_large = self.videofilepixbuf
             else:
                 desiredheight = int(64 / float(video[0].dar))
                 thumbnail = pixbuf.scale_simple(64,
                         desiredheight, gtk.gdk.INTERP_BILINEAR)
+                desiredheight = int(96 / float(video[0].dar))
+                thumbnail_large = pixbuf.scale_simple(96,
+                        desiredheight, gtk.gdk.INTERP_BILINEAR)
         else:
             if video:
                 thumbnail = self.videofilepixbuf
+                thumbnail_large = self.videofilepixbuf
             else:
                 thumbnail = self.audiofilepixbuf
+                thumbnail_large = self.audiofilepixbuf
 
         if not factory.duration or factory.duration == gst.CLOCK_TIME_NONE:
             duration = ''
         else:
             duration = beautify_length(factory.duration)
 
+        short_text = None
+        uni = unicode(factory_name(factory), 'utf-8')
+
+        if len(uni) > 34:
+            short_uni = uni[0:29]
+            short_uni += unicode('...')
+            short_text = short_uni.encode('utf-8')
+        else:
+            short_text = factory_name(factory)
+
         self.storemodel.append([thumbnail,
+            thumbnail_large,
             beautify_factory(factory),
             factory,
             factory.uri,
             duration,
-            factory_name(factory)])
-        self._displayTreeView()
+            factory_name(factory),
+            short_text])
+        self._displayClipView()
 
     # sourcelist callbacks
 
@@ -450,7 +546,7 @@ class SourceList(gtk.VBox, Loggable):
                 model.remove(row.iter)
                 break
         if not len(model):
-            self._displayTreeView(False)
+            self._displayHelpText()
 
     def _discoveryErrorCb(self, unused_sourcelist, uri, reason, extra):
         """ The given uri isn't a media file """
@@ -507,21 +603,26 @@ class SourceList(gtk.VBox, Loggable):
         self._importDialog = None
 
     def _removeSources(self):
-        tsel = self.treeview.get_selection()
-        if tsel.count_selected_rows() < 1:
+        model = self.storemodel
+        paths = self.getSelectedPaths()
+        if paths == None or paths < 1:
             return
-        model, selected = tsel.get_selected_rows()
-        # Sort the list in reverse order so we remove from
-        # the end and make sure that the paths are always valid
-        selected.sort(reverse=True)
-        for path in selected:
-            uri = model[path][COL_URI]
+        # use row references so we don't have to care if a path has been removed
+        rows = []
+        for path in paths:
+            row = gtk.TreeRowReference(model, path)
+            rows.append(row)
+
+        self.app.action_log.begin("remove clip from source list")
+        for row in rows:
+            uri = model[row.get_path()][COL_URI]
             self.app.current.sources.removeUri(uri)
+        self.app.action_log.commit()
 
     ## UI Button callbacks
 
-    def _addButtonClickedCb(self, unused_widget=None):
-        """ called when a user clicks on the add button """
+    def _importButtonClickedCb(self, unused_widget=None):
+        """ Called when a user clicks on the import button """
         self.showImportSourcesDialog()
 
     def _removeButtonClickedCb(self, unused_widget=None):
@@ -531,7 +632,8 @@ class SourceList(gtk.VBox, Loggable):
     def _playButtonClickedCb(self, unused_widget):
         """ Called when a user clicks on the play button """
         # get the selected filesourcefactory
-        model, paths = self.treeview.get_selection().get_selected_rows()
+        paths = self.getSelectedPaths()
+        model = self.storemodel
         if len(paths) < 1:
             return
         path = paths[0]
@@ -539,30 +641,98 @@ class SourceList(gtk.VBox, Loggable):
         self.debug("Let's play %s", factory.uri)
         self.emit('play', factory)
 
+    def _treeViewMenuItemToggledCb(self, unused_widget):
+        if self.treeview_menuitem.get_active():
+            show = SHOW_TREEVIEW
+        else:
+            show = SHOW_ICONVIEW
+        self._setClipView(show)
+
     _dragStarted = False
+    _dragSelection = False
     _dragButton = None
     _dragX = 0
     _dragY = 0
     _ignoreRelease = False
 
-    def _rowUnderMouseSelected(self, treeview, event):
-        result = treeview.get_path_at_pos(int(event.x), int(event.y))
+    def _rowUnderMouseSelected(self, view, event):
+        result = view.get_path_at_pos(int(event.x), int(event.y))
         if result:
             path = result[0]
-            selection = treeview.get_selection()
+            if isinstance(view, gtk.TreeView):
+                selection = view.get_selection()
 
-            return selection.path_is_selected(path) and selection.count_selected_rows() > 1
+                return selection.path_is_selected(path) and selection.count_selected_rows() > 0
+            elif isinstance(view, gtk.IconView):
+                selection = view.get_selected_items()
+
+                return view.path_is_selected(path) and len(selection)
+            else:
+                assert False
 
         return False
 
-    def _nothingUnderMouse(self, treeview, event):
-        return not bool(treeview.get_path_at_pos(int(event.x), int(event.y)))
+    def _nothingUnderMouse(self, view, event):
+        return not bool(view.get_path_at_pos(int(event.x), int(event.y)))
+
+    def _viewShowPopup(self, view, event):
+        if view != None and self._rowUnderMouseSelected(view, event):
+            self.popup_remitem.set_sensitive(True)
+            self.popup_playmenuitem.set_sensitive(True)
+        elif view != None and (not self._nothingUnderMouse(view, event)):
+            if not event.state & (gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK):
+                self._viewUnselectAll()
+            elif self.clip_view == SHOW_TREEVIEW and self._viewHasSelection() \
+                    and (event.state & gtk.gdk.SHIFT_MASK):
+                selection = self.treeview.get_selection()
+                start_path = self._viewGetFirstSelected()
+                end_path = self._viewGetPathAtPos(event)
+                self._viewUnselectAll()
+                selection.select_range(start_path, end_path)
+
+            self._viewSelectPath(self._viewGetPathAtPos(event))
+            self.popup_remitem.set_sensitive(True)
+            self.popup_playmenuitem.set_sensitive(True)
+        else:
+            self.popup_remitem.set_sensitive(False)
+            self.popup_playmenuitem.set_sensitive(False)
+
+        self.popup.popup(None, None, None, event.button, event.time)
+
+    def _viewGetFirstSelected(self):
+        paths = self.getSelectedPaths()
+        return paths[0]
+
+    def _viewHasSelection(self):
+        paths = self.getSelectedPaths()
+        return bool(len(paths))
+
+    def _viewGetPathAtPos(self, event):
+        if self.clip_view == SHOW_TREEVIEW:
+            pathinfo = self.treeview.get_path_at_pos(int(event.x), int(event.y))
+            return pathinfo[0]
+        elif self.clip_view == SHOW_ICONVIEW:
+            return self.iconview.get_path_at_pos(int(event.x), int(event.y))
+
+    def _viewSelectPath(self, path):
+        if self.clip_view == SHOW_TREEVIEW:
+            selection = self.treeview.get_selection()
+            selection.select_path(path)
+        elif self.clip_view == SHOW_ICONVIEW:
+            self.iconview.select_path(path)
+
+    def _viewUnselectAll(self):
+        if self.clip_view == SHOW_TREEVIEW:
+            selection = self.treeview.get_selection()
+            selection.unselect_all()
+        elif self.clip_view == SHOW_ICONVIEW:
+            self.iconview.unselect_all()
 
     def _treeViewButtonPressEventCb(self, treeview, event):
         chain_up = True
 
         if event.button == 3:
-            self.popup.popup(None, None, None, event.button, event.time)
+            self._viewShowPopup(treeview, event)
             chain_up = False
 
         else:
@@ -571,6 +741,7 @@ class SourceList(gtk.VBox, Loggable):
                 chain_up = not self._rowUnderMouseSelected(treeview, event)
 
             self._dragStarted = False
+            self._dragSelection = False
             self._dragButton = event.button 
             self._dragX = int(event.x)
             self._dragY = int(event.y)
@@ -612,8 +783,8 @@ class SourceList(gtk.VBox, Loggable):
                     treeview.get_selection().select_path(path)
         return False
 
-    def _treeSelectionChanged(self, tsel):
-        if self.getSelectedItems():
+    def _viewSelectionChangedCb(self, unused):
+        if self._viewHasSelection():
             self.selection_actions.set_sensitive(True)
         else:
             self.selection_actions.set_sensitive(False)
@@ -622,12 +793,72 @@ class SourceList(gtk.VBox, Loggable):
         factory = self.storemodel[path][COL_FACTORY]
         self.emit('play', factory)
 
+    def _iconViewMotionNotifyEventCb(self, iconview, event):
+        if not self._dragButton:
+            return True
+
+        if self._dragSelection:
+            return False
+
+        if self._nothingUnderMouse(iconview, event):
+            return True
+
+        if iconview.drag_check_threshold(self._dragX, self._dragY,
+            int(event.x), int(event.y)):
+            context = iconview.drag_begin(
+                [dnd.URI_TUPLE, dnd.FILESOURCE_TUPLE],
+                gtk.gdk.ACTION_COPY,
+                self._dragButton,
+                event)
+            self._dragStarted = True
+        return False
+
+    def _iconViewButtonPressEventCb(self, iconview, event):
+        chain_up = True
+
+        #popup menu
+        if event.button == 3:
+            self._viewShowPopup(iconview, event)
+            chain_up = False
+        else:
+            if not event.state & (gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK):
+                chain_up = not self._rowUnderMouseSelected(iconview, event)
+
+            self._dragStarted = False
+            self._dragSelection = self._nothingUnderMouse(iconview, event)
+            self._dragButton = event.button
+            self._dragX = int(event.x)
+            self._dragY = int(event.y)
+
+        if chain_up:
+            gtk.IconView.do_button_press_event(iconview, event)
+        else:
+            iconview.grab_focus()
+
+        self._ignoreRelease = chain_up
+
+        return True
+
+    def _iconViewButtonReleaseCb(self, iconview, event):
+        if event.button == self._dragButton:
+            self._dragButton = None
+            self._dragSelection = False
+            if (not self._ignoreRelease) and (not self._dragStarted):
+                iconview.unselect_all()
+                path = iconview.get_path_at_pos(int(event.x), int(event.y))
+                if path:
+                    iconview.select_path(path)
+        return False
+
+    def _textBoxButtonPressEventCb(self, textbox, event):
+        if event.button == 3:
+            self._viewShowPopup(None, event)
+
     def _newProjectCreatedCb(self, app, project):
         # clear the storemodel
         self.storemodel.clear()
         self._connectToProject(project)
 
-
     def _newProjectLoadingCb(self, unused_pitivi, uri):
         if not self.infostub.showing:
             self.pack_start(self.infostub, expand=False)
@@ -670,25 +901,43 @@ class SourceList(gtk.VBox, Loggable):
                 directories = [incoming]
         if directories:
             self.addFolders(directories)
+
         try:
-            self.addUris(filenames)
+            self.addUris([quote_uri(uri) for uri in filenames])
         except SourceListError:
             # filenames already present in the sourcelist
             pass
 
-    def _dndTreeBeginCb(self, unused_widget, context):
+    #used with TreeView and IconView
+    def _dndDragBeginCb(self, view, context):
         self.info("tree drag_begin")
-        model, paths = self.treeview.get_selection().get_selected_rows()
+        paths = self.getSelectedPaths()
+
         if len(paths) < 1:
             context.drag_abort(int(time.time()))
         else:
-            row = model[paths[0]]
+            row = self.storemodel[paths[0]]
             context.set_icon_pixbuf(row[COL_ICON], 0, 0)
 
-    def getSelectedItems(self):
+    def getSelectedPaths(self):
         """ returns a list of selected items uri """
+        if self.clip_view == SHOW_TREEVIEW:
+            return self.getSelectedPathsTreeView()
+        elif self.clip_view == SHOW_ICONVIEW:
+            return self.getSelectedPathsIconView()
+
+    def getSelectedPathsTreeView(self):
         model, rows = self.treeview.get_selection().get_selected_rows()
-        return [model[path][COL_URI] for path in rows]
+        return rows
+
+    def getSelectedPathsIconView(self):
+        paths = self.iconview.get_selected_items()
+        paths.reverse()
+        return paths
+
+    def getSelectedItems(self):
+        return [self.storemodel[path][COL_URI] 
+            for path in self.getSelectedPaths()]
 
     def _dndDataGetCb(self, unused_widget, context, selection,
                       targettype, unused_eventtime):
@@ -724,17 +973,21 @@ class InfoStub(gtk.HBox, Loggable):
         self.set_spacing(6)
         anim = gtk.gdk.PixbufAnimation(get_pixmap_dir() + "/busy.gif")
         self.busyanim = gtk.image_new_from_animation(anim)
+        self.busyanim.show()
 
         self.erroricon = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
                                                   gtk.ICON_SIZE_SMALL_TOOLBAR)
+        self.erroricon.show()
 
         self.infolabel = gtk.Label(self._importingmessage)
         self.infolabel.set_alignment(0, 0.5)
+        self.infolabel.show()
 
         self.questionbutton = gtk.Button()
         self.questionbutton.set_image(gtk.image_new_from_stock(gtk.STOCK_INFO,
                                                                gtk.ICON_SIZE_SMALL_TOOLBAR))
         self.questionbutton.connect("clicked", self._questionButtonClickedCb)
+        self.questionbutton.show()
         self._questionshowing = False
 
         self.pack_start(self.busyanim, expand=False)
@@ -819,13 +1072,4 @@ class InfoStub(gtk.HBox, Loggable):
         self.hide()
         self.emit("remove-me")
 
-    def show(self):
-        self.log("showing")
-        self.show_all()
-        self.showing = True
-
-    def hide(self):
-        self.log("hiding")
-        gtk.VBox.hide(self)
-        self.showing = False
 gobject.type_register(SourceList)
