@@ -4,6 +4,7 @@
 #       utils.py
 #
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
+# Copyright (c) 2009, Alessandro Decina <alessandro.d@gmail.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -29,9 +30,13 @@ import os
 from pitivi.signalinterface import Signallable
 import pitivi.log.log as log
 from gettext import ngettext
+import cProfile
 
 UNKNOWN_DURATION = 2 ** 63 - 1
 
+def between(a, b, c):
+    return (a <= b) and (b <= c)
+
 def time_to_string(value):
     """
     Converts the given time in nanoseconds to a human readable string
@@ -240,6 +245,9 @@ def uri_is_reachable(uri):
     return os.path.isfile(gst.uri_get_location(uri))
 
 class PropertyChangeTracker(Signallable):
+
+    __signals__ = {}
+
     def __init__(self):
         self.properties = {}
         self.obj = None
@@ -281,18 +289,16 @@ class Seeker(Signallable):
         self.format = None
 
     def seek(self, position, format=gst.FORMAT_TIME, on_idle=False):
+        self.position = position
+        self.format = format
+
         if self.pending_seek_id is None:
-            self.position = position
-            self.format = format
             if on_idle:
                 gobject.idle_add(self._seekTimeoutCb)
             else:
                 self._seekTimeoutCb()
             self.pending_seek_id = self._scheduleSeek(self.timeout,
                     self._seekTimeoutCb)
-        else:
-            self.position = position
-            self.format = format
 
     def _scheduleSeek(self, timeout, callback):
         return gobject.timeout_add(timeout, callback)
@@ -430,3 +436,54 @@ def getNextObject(obj, objects, priority=-1, skip=None):
         next_obj_index += 1
 
     return None
+
+
+class CachedFactoryList(object):
+    def __init__(self, factoryFilter=None):
+        self._factoryFilter = factoryFilter
+        self._factories = None
+        self._registry = gst.registry_get_default()
+        self._registry.connect("feature-added", self._registryFeatureAddedCb)
+
+    def get(self):
+        if self._factories is None:
+            self._buildFactories()
+
+        return self._factories
+
+    def _buildFactories(self):
+        # build the cache
+        log.debug("utils", "Getting factories list")
+        factories = self._registry.get_feature_list(gst.ElementFactory)
+        if self._factoryFilter is not None:
+            log.debug("utils", "filtering")
+            factories = filter(self._factoryFilter, factories)
+
+        log.debug("utils", "Sorting by rank")
+        factories.sort(key=lambda factory: factory.get_rank(), reverse=True)
+        self._factories = factories
+        log.debug("utils", "Cached factories is now %r", self._factories)
+
+    def _registryFeatureAddedCb(self, registry, feature):
+        # invalidate the cache
+        log.warning("utils", "New feature added, invalidating cached factories")
+        self._factories = None
+
+def profile(func, profiler_filename="result.prof"):
+    import os.path
+    counter = 1
+    output_filename = profiler_filename
+    while os.path.exists(output_filename):
+        output_filename = profiler_filename + str(counter)
+        counter += 1
+
+    def _wrapper(*args,**kwargs):
+        local_func = func
+        cProfile.runctx("result = local_func(*args, **kwargs)", globals(), locals(),
+                        filename=output_filename)
+        return locals()["result"]
+
+    return _wrapper
+
+def formatPercent(value):
+    return "%3d%%" % (value * 100)
